"""
Enterprise Integrated Channel Cornerstone Django application initialization.
"""

from django.apps import AppConfig


class CornerstoneConfig(AppConfig):
    """
    Configuration for the Enterprise Integrated Channel Cornerstone Django application.
    """
    name = 'integrated_channels.cornerstone'
    verbose_name = "Enterprise Cornerstone Integration"
