# Generated by Django 2.2.19 on 2021-11-29 21:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0147_auto_20211129_1949'),
    ]

    operations = [
        migrations.AlterField(
            model_name='enterprisecustomer',
            name='enable_universal_link',
            field=models.BooleanField(default=False, help_text='Specifies whether universal link generation is enabled for customer. Managed via admin portal settings UI.'),
        ),
        migrations.AlterField(
            model_name='historicalenterprisecustomer',
            name='enable_universal_link',
            field=models.BooleanField(default=False, help_text='Specifies whether universal link generation is enabled for customer. Managed via admin portal settings UI.'),
        ),
    ]
