# Generated by Django 2.2.24 on 2021-10-14 14:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('integrated_channel', '0005_auto_20211005_1052'),
    ]

    operations = [
        migrations.AddField(
            model_name='contentmetadataitemtransmission',
            name='deleted_at',
            field=models.DateTimeField(blank=True, help_text='Date when the content transmission was deleted', null=True),
        ),
    ]
