# Generated by Django 3.2.12 on 2022-03-24 15:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cornerstone', '0015_auto_20220302_2231'),
    ]

    operations = [
        migrations.AddField(
            model_name='cornerstonelearnerdatatransmissionaudit',
            name='enterprise_customer_uuid',
            field=models.UUIDField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='cornerstonelearnerdatatransmissionaudit',
            name='instructor_name',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AddField(
            model_name='cornerstonelearnerdatatransmissionaudit',
            name='plugin_configuration_id',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='cornerstonelearnerdatatransmissionaudit',
            name='total_hours',
            field=models.FloatField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='cornerstonelearnerdatatransmissionaudit',
            name='completed_timestamp',
            field=models.DateTimeField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='cornerstonelearnerdatatransmissionaudit',
            name='course_completed',
            field=models.BooleanField(default=True),
        ),
        migrations.AlterField(
            model_name='cornerstonelearnerdatatransmissionaudit',
            name='course_id',
            field=models.CharField(max_length=255),
        ),
    ]
