# Generated by Django 2.2.17 on 2021-01-11 12:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', 'unique_constraints_pending_users'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecustomer',
            name='sender_alias',
            field=models.CharField(blank=True, help_text='Specifies the sender alias which will be used in enterprise customer Assign/Remind/Revoke emails.', max_length=255, null=True),
        ),
        migrations.AddField(
            model_name='historicalenterprisecustomer',
            name='sender_alias',
            field=models.CharField(blank=True, help_text='Specifies the sender alias which will be used in enterprise customer Assign/Remind/Revoke emails.', max_length=255, null=True),
        ),
    ]
